use std::{error::Error, io, iter::zip};

use crossterm::{terminal, ExecutableCommand};
use ratatui::{
    layout::{Constraint, Direction, Layout},
    prelude::CrosstermBackend,
    style::{Color, Style},
    widgets::{Block, Borders, List, ListItem, Paragraph, Wrap},
    Terminal,
};

use crate::{
    event_handler::event_key_handler,
    state::ApplicationState,
    widgets::blocks::{basic_block, highlighted_border_block, highlighted_item_block},
};

pub fn run(app_state: &mut ApplicationState) -> Result<(), Box<dyn Error>> {
    // Set up terminal
    let mut stdout = io::stdout();
    terminal::enable_raw_mode()?;
    stdout.execute(terminal::Clear(terminal::ClearType::All))?;
    stdout.execute(terminal::EnterAlternateScreen)?;

    // Initialize the terminal with the Crossterm backend
    let backend = CrosstermBackend::new(stdout);
    let mut terminal = Terminal::new(backend)?;

    // Start the main loop for rendering
    loop {
        terminal.draw(|f| {
            // Define the layout with the specified number of columns
            let size = f.area();
            let constraints: Vec<Constraint> =
                vec![Constraint::Percentage(100 / app_state.columns as u16); app_state.columns]
                    .into_iter()
                    .collect();

            let chunks = Layout::default()
                .direction(Direction::Horizontal)
                .constraints::<&Vec<Constraint>>(constraints.as_ref())
                .split(size);

            // Render a item blocks in each column
            for (i, chunk) in chunks.iter().enumerate() {
                let block = if i == app_state.selected_col {
                    highlighted_border_block(i)
                } else {
                    basic_block(Some(i))
                };
                f.render_widget(block, *chunk);

                let nitems = app_state.col_n_items(i);
                if nitems == 0 {
                    continue;
                }
                let constraints: Vec<Constraint> =
                    vec![Constraint::Max(5); nitems].into_iter().collect();
                let sub_chunks = Layout::default()
                    .direction(Direction::Vertical)
                    .constraints::<&Vec<Constraint>>(constraints.as_ref())
                    .margin(1)
                    .split(*chunk);
                for ((task, highlight), sub_chunk) in
                    zip(app_state.get_col_slice(i), sub_chunks.iter())
                {
                    let style = if highlight {
                        Style::default().bg(Color::Blue).fg(Color::DarkGray)
                    } else {
                        Style::default()
                    };
                    let paragraph = Paragraph::new(task.title.clone())
                        .block(basic_block(None))
                        .style(style);
                    f.render_widget(paragraph, *sub_chunk);
                }
            }
        })?;

        // Handle events such as key presses
        event_key_handler(app_state)?;
        if app_state.should_quit {
            break;
        };
    }

    Ok(())
}
